#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include <arpa/inet.h>

#include "../inc/lc6_config.h"
#include "../inc/lc6_time.h"
#include "../inc/lc6_bootstrap.h"

const char bootlist[4][50] = {
	"127.0.0.1",
	"2001:1a8f:ffff::abcd",
	"blah blah blah lol",
	"yep: not working ;-)"
};

int main(int argc, char **argv) {
	LC6_CTX *conf = NULL;


	if ( argc == 2 ) {
		int i = 0;;

		printf("Loading configuration files...\n");
		conf = lc6config_load(argv[1], "");

		for(i=0; i<sizeof(bootlist)/sizeof(bootlist[0]); i++) {
			char *str = NULL;
			struct in_addr addr;
			struct in6_addr addr6;
			LC6_BOOTSTRAP *bs = malloc(sizeof(LC6_BOOTSTRAP));

			assert(bs);
			memset(bs, '\0', sizeof(LC6_BOOTSTRAP));

			printf("Parsing potential IP address '%s'\n", bootlist[i]);

			bs->prev = NULL;
			bs->next = NULL;
			bs->last_contact = lc6time_get();

			printf("Decoding... ");

			if ( inet_pton(AF_INET, bootlist[i], &addr) == 1 ) {
				printf("IPv4\n");
				bs->af = 4;
				bs->addr.inet = addr.s_addr;
			}
			else if ( inet_pton(AF_INET6, bootlist[i], &addr6) == 1) {
				printf("IPv6\n");
				bs->af = 6;
				memcpy(bs->addr.inet6, &addr6, sizeof(bs->addr.inet6));
			}
			else {
				printf("Failed\n");
				free(bs);
				bs = NULL;
			}

			str = lc6bootstrap_getip(bs);
			printf("The result is : %s\n", str ? str : "NULL");
			if ( str )
				free(str);

			if ( bs )
				lc6config_bootstrap_add(conf, bs);

		}

		lc6config_save(conf, "");
		lc6config_free(conf);
	}
	else {
		fprintf(stderr,"\
This is a helper programm for libchat6.\n\
It will initialise a bootstrap.bin configuration file\n\
containing a list of bootstrap clients hardcoded at the top of\n\
bootstrap.c. The default password for the bootstrap.bin file\n\
is an empty string.\n\n");

		fprintf(stderr,"Usage: %s <path to config dir>\n", argv[0]);
		return -1;
	}


	return 0;
}
