#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <sodium.h>

#include "../inc/libchat6.h"

#include "../inc/lc6_helpers.h"
#include "../inc/lc6_crypto.h"
#include "../inc/lc6_user.h"


int main(int argc, char **argv) {
	unsigned char data[1024];
	unsigned char data2[1024];
	unsigned char *ptr_data = NULL;
	unsigned char *ptr_data2 = NULL;
	unsigned char *ptr_nonce = NULL;
	int i, len;
	LC6_USER *user;

	lc6helpers_banner("Generating random bytes...");
	for(i=0; i<8; i++) {
		printf("random bytes... ");
		lc6crypto_random(data, 16);
		lc6helpers_printhex(data, 16);
		printf("\n");
	}
	
	lc6helpers_banner("Generating SHA256 hashs...");
	for(i=0; i<8; i++) {
		unsigned char *hash;
		lc6crypto_random(data, 16);
		hash = lc6crypto_hash(data, 16);
		printf("random: ");
		lc6helpers_printhex(data, 16);
		printf(", hash: ");
		lc6helpers_printhex(hash, 32);
		printf("\n");
		free(hash);
	}


	lc6helpers_banner("Generating new user and keys...");
	user = lc6user_create();

	strcpy(user->nickname,"Päscu");

	printf("Private key\n");
	lc6helpers_printhex(user->priv_key, crypto_box_SECRETKEYBYTES);
	printf("\n");
	printf("Public key\n");
	lc6helpers_printhex(user->pub_key, crypto_box_PUBLICKEYBYTES);

	lc6helpers_banner("Public key encrypt, private key decrypt");

	strcpy(data,"This is a test message to be encrypted. It may contain UTF-8 chars like 🤪");
	printf("plain data: %s\n", data);


	len = lc6crypto_public_encrypt(data, strlen(data), user->priv_key, user->pub_key, &ptr_data, &ptr_nonce);
	printf("encrypted : ");
	lc6helpers_printhex(ptr_data, len);
	printf("\n");

	len = lc6crypto_private_decrypt(ptr_data, len, user->priv_key, user->pub_key, &ptr_data, ptr_nonce);
	printf("decrypted : %s\n", ptr_data);

	free(ptr_data);
	free(ptr_nonce);
	free(ptr_data2);

	lc6helpers_banner("Private key encrypt, public key decrypt");

	strcpy(data,"This is a test message to be encrypted. It may contain UTF-8 chars like 🤪");
	printf("plain data: %s\n", data);


	len = lc6crypto_private_encrypt(data, strlen(data), user->priv_key, user->pub_key, &ptr_data, &ptr_nonce);
	printf("encrypted : ");
	lc6helpers_printhex(ptr_data, len);
	printf("\n");

	len = lc6crypto_public_decrypt(ptr_data, len, user->priv_key, user->pub_key, &ptr_data, ptr_nonce);
	printf("decrypted : %s\n", ptr_data);

	free(ptr_data);
	free(ptr_nonce);
	free(ptr_data2);

	lc6user_free(NULL, user);
	user = NULL;

	return 0;
}
